﻿using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using System;
using System.IO;
using System.Reflection;
using System.Runtime.Loader;

namespace Falcon.SugarApi.Plugin
{
    /// <summary>
    /// 查找插件服务
    /// </summary>
    public static class FindPluginService
    {
        /// <summary>
        /// 查找并增加插件服务
        /// </summary>
        /// <param name="assemblyFile">程序集名称</param>
        /// <param name="services">服务集合</param>
        /// <param name="configuration">配置</param>
        public static void FindAddPluginService(string assemblyFile,IServiceCollection services,IConfiguration configuration) {
            if(!File.Exists(assemblyFile)) {
                throw new FileNotFoundException($"插件文件没有找到：{assemblyFile}");
            }
            //var doMain = AppDomain.CurrentDomain;
            //var assembly = doMain.Load(assemblyFile);

            var dm = new AssemblyLoadContext(null,false);
            var assembly = dm.LoadFromAssemblyPath(assemblyFile);

            FindAddPluginService(assembly,services,configuration);
        }
        /// <summary>
        /// 查找并增加插件服务
        /// </summary>
        /// <param name="assembly">程序集</param>
        /// <param name="services">服务集合</param>
        /// <param name="configuration">配置</param>
        public static void FindAddPluginService(Assembly assembly,IServiceCollection services,IConfiguration configuration) {
            var name = typeof(IServicePlugin).FullName;
            if(name == null) {
                return;
            }
            foreach(Type type in assembly.GetTypes()) {
                if(type == null || !type.IsPublic) {
                    continue;
                }
                if(type.GetInterface(name) == null) {
                    continue;
                }
                var tname = type.FullName;
                if(tname == null) {
                    continue;
                }
                var obj = type.Assembly.CreateInstance(tname) as IServicePlugin;
                if(obj == null) {
                    continue;
                }
                obj.AddServices(services,configuration);
            }
        }
    }
}
